@@ -78,6 +78,9 @@
 #define CFG_SNI_NOMATCH_ABORT "sni-nomatch-abort"
 #define CFG_OCSP_DIR "ocsp-dir"
 #define CFG_TLS_PROTOS "tls-protos"
+#define CFG_PARAM_TLS_PROTOS 11018
+#define CFG_DBG_LISTEN "dbg-listen"
+#define CFG_PARAM_DBG_LISTEN 11019
 #ifdef TCP_FASTOPEN_WORKS
 	#define CFG_TFO "enable-tcp-fastopen"
 #endif
@@ -193,6 +196,7 @@ config_new(void)
 	r->WRITE_PROXY_LINE_V2		= 0;
 	r->PROXY_TLV			= 1;
 	r->PROXY_AUTHORITY		= 1;
+	r->PROXY_CLIENT_CERT		= 0;
 	r->PROXY_PROXY_LINE		= 0;
 	r->ALPN_PROTOS			= NULL;
 	r->ALPN_PROTOS_LV		= NULL;
@@ -341,9 +345,12 @@ config_param_val_bool(char *val, int *res)
 
 	if (strcasecmp(val, CFG_BOOL_ON) == 0 || strcasecmp(val, "yes") == 0 ||
 	    strcasecmp(val, "y") == 0 || strcasecmp(val, "true") == 0 ||
-	    strcasecmp(val, "t") == 0 || strcasecmp(val, "1") == 0) {
+	    strcasecmp(val, "t") == 0 || strcasecmp(val, "1") == 0)
 		*res = 1;
-	}
+	else if (strcasecmp(val, "off") == 0 || strcasecmp(val, "no") == 0
+	    || strcasecmp(val, "n") == 0 || strcasecmp(val, "false") == 0
+	    || strcasecmp(val, "f") == 0 || strcasecmp(val, "0") == 0)
+		*res = 0;
 
 	return (1);
 }
@@ -831,7 +838,7 @@ front_arg_add(hitch_config *cfg, struct front_arg *fa)
 }
 
 int
-config_param_validate(char *k, char *v, hitch_config *cfg,
+config_param_validate(const char *k, char *v, hitch_config *cfg,
     char *file, int line)
 {
 	int r = 1;
@@ -1030,6 +1037,18 @@ config_param_validate(char *k, char *v, hitch_config *cfg,
 	} else if (strcmp(k, CFG_TFO) == 0) {
 		config_param_val_bool(v, &cfg->TFO);
 #endif
+	} else if (strcmp(k, CFG_TLS_PROTOS) == 0) {
+		cfg->SELECTED_TLS_PROTOS = 0;
+#define TLS_PROTO(u, i, s)				\
+		if (strcasestr(v, s))			\
+			cfg->SELECTED_TLS_PROTOS |= i;
+#include "tls_proto_tbl.h"
+		if (cfg->SELECTED_TLS_PROTOS == 0) {
+			config_error_set("Invalid 'tls-protos' option '%s'", v);
+			return (1);
+		}
+	} else if (strcmp(k, CFG_DBG_LISTEN) == 0) {
+		config_assign_str(&cfg->DEBUG_LISTEN_ADDR, v);
 	} else {
 		fprintf(
 			stderr,
@@ -1230,100 +1249,175 @@ config_print_usage_fd(char *prog, FILE *out)
 	fprintf(out, "This is hitch, The Scalable TLS Unwrapping Daemon.\n\n");
 	fprintf(out, "CONFIGURATION:\n");
 	fprintf(out, "\n");
-	fprintf(out, "        --config=FILE      Load configuration from specified file.\n");
+	fprintf(out, "\t--config=FILE\n");
+	fprintf(out, "\t\tLoad configuration from specified file.\n");
 	fprintf(out, "\n");
 	fprintf(out, "ENCRYPTION METHODS:\n");
 	fprintf(out, "\n");
-	fprintf(out, "      --tls                   TLSv1 (default. No SSLv3)\n");
-	fprintf(out, "      --ssl                   SSLv3 (enables SSLv3)\n");
-	fprintf(out, "  -c  --ciphers=SUITE         Sets allowed ciphers (Default: \"%s\")\n", config_disp_str(cfg->CIPHERS_TLSv12));
-	fprintf(out, "  -e  --ssl-engine=NAME       Sets OpenSSL engine (Default: \"%s\")\n", config_disp_str(cfg->ENGINE));
-	fprintf(out, "  -O  --prefer-server-ciphers Prefer server list order\n");
+	fprintf(out, "\t--tls-protos=LIST\n");
+	fprintf(out, "\t\tSpecifies which SSL/TLS protocols to use.\n");
+	fprintf(out, "\t\tAvailable tokens are SSLv3, TLSv1.0, TLSv1.1\n");
+	fprintf(out, "\t\tTLSv1.2 and TLSv1.3. (Default: \"TLSv1.2 TLSv1.3\"\n)");
+	fprintf(out, "\t-c  --ciphers=SUITE\n");
+	fprintf(out, "\t\tSets allowed ciphers (Default: \"%s\")\n",
+	    config_disp_str(cfg->CIPHERS_TLSv12));
+	fprintf(out, "\t-e  --ssl-engine=NAME\n");
+	fprintf(out, "\t\tSets OpenSSL engine (Default: \"%s\")\n",
+	    config_disp_str(cfg->ENGINE));
+	fprintf(out, "\t-O  --prefer-server-ciphers[=on|off]\n");
+	fprintf(out, "\t\tPrefer server list order (Default: \"%s\")\n",
+	    config_disp_bool(cfg->PREFER_SERVER_CIPHERS));
 	fprintf(out, "\n");
 	fprintf(out, "SOCKET:\n");
 	fprintf(out, "\n");
-	fprintf(out, "      --client                Enable client proxy mode\n");
-	fprintf(out, "  -b  --backend=[HOST]:PORT   Backend [connect] (default is \"%s\")\n", config_disp_hostport(cfg->BACK_IP, cfg->BACK_PORT));
-	fprintf(out, "                              The -b argument can also take a UNIX domain socket path\n");
-	fprintf(out, "                              E.g. --backend=\"/path/to/sock\"\n");
-	fprintf(out, "  -f  --frontend=[HOST]:PORT[+CERT]    Frontend [bind] (default is \"%s\")\n", config_disp_hostport(cfg->LISTEN_DEFAULT->ip, cfg->LISTEN_DEFAULT->port));
-	fprintf(out, "                                (Note: brackets are mandatory in endpoint specifiers.)\n");
-	fprintf(out, "      --recv-bufsize=SIZE    Receive buffer size on client socket (Default: %d)\n", cfg->RECV_BUFSIZE);
-	fprintf(out, "      --send-bufsize=SIZE    Send buffer size on client socket (Default: %d)\n", cfg->SEND_BUFSIZE);
+	fprintf(out, "\t--client\n");
+	fprintf(out, "\t\tEnable client proxy mode\n");
+	fprintf(out, "\t-b  --backend=[HOST]:PORT\n");
+	fprintf(out, "\t\tBackend endpoint (default is \"%s\")\n",
+	    config_disp_hostport(cfg->BACK_IP, cfg->BACK_PORT));
+	fprintf(out,
+	    "\t\tThe -b argument can also take a UNIX domain socket path\n");
+	fprintf(out, "\t\tE.g. --backend=\"/path/to/sock\"\n");
+	fprintf(out, "\t-f  --frontend=[HOST]:PORT[+CERT]\n");
+	fprintf(out, "\t\tFrontend listen endpoint (default is \"%s\")\n",
+	    config_disp_hostport(cfg->LISTEN_DEFAULT->ip,
+		cfg->LISTEN_DEFAULT->port));
+	fprintf(out,
+	    "\t\t(Note: brackets are mandatory in endpoint specifiers.)\n");
+	fprintf(out, "\t--recv-bufsize=SIZE\n");
+	fprintf(out, "\t\tReceive buffer size on client socket (Default: %d)\n",
+	    cfg->RECV_BUFSIZE);
+	fprintf(out, "\t--send-bufsize=SIZE\n");
+	fprintf(out, "\t\tSend buffer size on client socket (Default: %d)\n",
+	    cfg->SEND_BUFSIZE);
 
 #ifdef USE_SHARED_CACHE
 	fprintf(out, "\n");
-	fprintf(out, "  -U  --shared-cache-listen=[HOST]:PORT\n");
-	fprintf(out, "                              Accept cache updates on UDP (Default: \"%s\")\n", config_disp_hostport(cfg->SHCUPD_IP, cfg->SHCUPD_PORT));
-	fprintf(out, "                              NOTE: This option requires enabled SSL session cache.\n");
-	fprintf(out, "  -P  --shared-cache-peer=[HOST]:PORT\n");
-	fprintf(out, "                              Send cache updates to specified peer\n");
-	fprintf(out, "                              NOTE: This option can be specified multiple times.\n");
-	fprintf(out, "  -M  --shared-cache-if=IFACE[,TTL]\n");
-	fprintf(out, "                              Force iface and ttl to receive and send multicast updates\n");
+	fprintf(out, "\t-U  --shared-cache-listen=[HOST]:PORT\n");
+	fprintf(out, "\t\tAccept cache updates on UDP (Default: \"%s\")\n",
+	    config_disp_hostport(cfg->SHCUPD_IP, cfg->SHCUPD_PORT));
+	fprintf(out,
+	    "\t\tNOTE: This option requires enabled SSL session cache.\n");
+	fprintf(out, "\t-P  --shared-cache-peer=[HOST]:PORT\n");
+	fprintf(out, "\t\tSend cache updates to specified peer\n");
+	fprintf(out,
+	    "\t\tNOTE: This option can be specified multiple times.\n");
+	fprintf(out, "\t-M  --shared-cache-if=IFACE[,TTL]\n");
+	fprintf(out,
+	    "\t\tForce iface and ttl to receive and send multicast updates\n");
 #endif
 
 	fprintf(out, "\n");
 	fprintf(out, "PERFORMANCE:\n");
 	fprintf(out, "\n");
-	fprintf(out, "  -n  --workers=NUM          Number of worker processes (Default: %ld)\n", cfg->NCORES);
-	fprintf(out, "  -B  --backlog=NUM          Set listen backlog size (Default: %d)\n", cfg->BACKLOG);
-	fprintf(out, "  -k  --keepalive=SECS       TCP keepalive on client socket (Default: %d)\n", cfg->TCP_KEEPALIVE_TIME);
-	fprintf(out, "  -R  --backend-refresh=SECS Periodic backend IP lookup, 0 to disable (Default: %d)\n", cfg->BACKEND_REFRESH_TIME);
-
+	fprintf(out, "\t-n  --workers=NUM\n");
+	fprintf(out, "\t\tNumber of worker processes (Default: %ld)\n",
+	    cfg->NCORES);
+	fprintf(out, "\t-B  --backlog=NUM\n");
+	fprintf(out, "\t\tSet listen backlog size (Default: %d)\n", cfg->BACKLOG);
+	fprintf(out, "\t-k  --keepalive=SECS\n");
+	fprintf(out, "\t\tTCP keepalive on client socket (Default: %d)\n",
+	    cfg->TCP_KEEPALIVE_TIME);
+	fprintf(out, "\t-R  --backend-refresh=SECS\n");
+	fprintf(out, "\t\tPeriodic backend IP lookup, 0 to disable (Default: %d)\n",
+	    cfg->BACKEND_REFRESH_TIME);
 
 #ifdef USE_SHARED_CACHE
-	fprintf(out, "  -C  --session-cache=NUM    Enable and set SSL session cache to specified number\n");
-	fprintf(out, "                             of sessions (Default: %d)\n", cfg->SHARED_CACHE);
+	fprintf(out, "\t-C  --session-cache=NUM\n");
+	fprintf(out,
+	    "\t\tEnable and set SSL session cache to specified number\n");
+	fprintf(out, "\t\tof sessions (Default: %d)\n", cfg->SHARED_CACHE);
 #endif
 #ifdef TCP_FASTOPEN_WORKS
-	fprintf(out, "      --enable-tcp-fastopen  Enable client-side TCP Fast Open. (Default: %s)\n", config_disp_bool(cfg->TFO));
+	fprintf(out, "\t--enable-tcp-fastopen[=on|off]\n");
+	fprintf(out, "\t\tEnable client-side TCP Fast Open. (Default: %s)\n",
+	    config_disp_bool(cfg->TFO));
 #endif
-
 	fprintf(out, "\n");
 	fprintf(out, "SECURITY:\n");
 	fprintf(out, "\n");
-	fprintf(out, "  -r  --chroot=DIR           Sets chroot directory (Default: \"%s\")\n", config_disp_str(cfg->CHROOT));
-	fprintf(out, "  -u  --user=USER            Set uid/gid after binding the socket (Default: \"%s\")\n", config_disp_uid(cfg->UID));
-	fprintf(out, "  -g  --group=GROUP          Set gid after binding the socket (Default: \"%s\")\n", config_disp_gid(cfg->GID));
+	fprintf(out, "\t-r  --chroot=DIR\n");
+	fprintf(out, "\t\tSets chroot directory (Default: \"%s\")\n",
+	    config_disp_str(cfg->CHROOT));
+	fprintf(out, "\t-u  --user=USER\n ");
+	fprintf(out,
+	    "\t\tSet uid/gid after binding the socket (Default: \"%s\")\n",
+	    config_disp_uid(cfg->UID));
+	fprintf(out, "\t-g  --group=GROUP\n");
+	fprintf(out, "\t\tSet gid after binding the socket (Default: \"%s\")\n",
+	    config_disp_gid(cfg->GID));
 	fprintf(out, "\n");
 	fprintf(out, "LOGGING:\n");
-	fprintf(out, "  -q  --quiet                Be quiet; emit only error messages (deprecated, use 'log-level')\n");
-	fprintf(out, "  -L  --log-level=NUM        Log level. 0=silence, 1=err, 2=info/debug (Default: %d)\n",
+	fprintf(out, "\t-q  --quiet[=on|off]\n");
+	fprintf(out, "\t\tBe quiet; emit only error messages "
+	    "(deprecated, use 'log-level')\n");
+	fprintf(out, "\t-L  --log-level=NUM\n");
+	fprintf(out, "\t\tLog level. 0=silence, 1=err, 2=info/debug (Default: %d)\n",
 		cfg->LOG_LEVEL);
-	fprintf(out, "  -l  --log-filename=FILE    Send log message to a logfile instead of stderr/stdout\n");
-	fprintf(out, "  -s  --syslog               Send log message to syslog in addition to stderr/stdout\n");
-	fprintf(out, "      --syslog-facility=FACILITY    Syslog facility to use (Default: \"%s\")\n", config_disp_log_facility(cfg->SYSLOG_FACILITY));
+	fprintf(out, "\t-l  --log-filename=FILE \n");
+	fprintf(out,
+	    "\t\tSend log message to a logfile instead of stderr/stdout\n");
+	fprintf(out, "\t-s  --syslog[=on|off]   \n");
+	fprintf(out,
+	    "\t\tSend log message to syslog in addition to stderr/stdout\n");
+	fprintf(out, "\t--syslog-facility=FACILITY\n");
+	fprintf(out, "\t\tSyslog facility to use (Default: \"%s\")\n",
+	    config_disp_log_facility(cfg->SYSLOG_FACILITY));
 	fprintf(out, "\n");
 	fprintf(out, "OTHER OPTIONS:\n");
-	fprintf(out, "      --daemon               Fork into background and become a daemon (Default: %s)\n", config_disp_bool(cfg->DAEMONIZE));
-	fprintf(out, "      --write-ip             Write 1 octet with the IP family followed by the IP\n");
-	fprintf(out, "                             address in 4 (IPv4) or 16 (IPv6) octets little-endian\n");
-	fprintf(out, "                             to backend before the actual data\n");
-	fprintf(out, "                             (Default: %s)\n", config_disp_bool(cfg->WRITE_IP_OCTET));
-	fprintf(out, "      --write-proxy-v1       Write HaProxy's PROXY v1 (IPv4 or IPv6) protocol line\n" );
-	fprintf(out, "                             before actual data\n");
-	fprintf(out, "                             (Default: %s)\n", config_disp_bool(cfg->WRITE_PROXY_LINE_V1));
-	fprintf(out, "      --write-proxy-v2       Write HaProxy's PROXY v2 binary (IPv4 or IPv6)  protocol line\n" );
-	fprintf(out, "                             before actual data\n");
-	fprintf(out, "                             (Default: %s)\n", config_disp_bool(cfg->WRITE_PROXY_LINE_V2));
-	fprintf(out, "      --write-proxy          Equivalent to --write-proxy-v2. For PROXY version 1 use\n");
-	fprintf(out, "                              --write-proxy-v1 explicitly\n");
-	fprintf(out, "      --proxy-proxy          Proxy HaProxy's PROXY (IPv4 or IPv6) protocol line\n" );
-	fprintf(out, "                             before actual data (PROXY v1 only)\n");
-	fprintf(out, "                             (Default: %s)\n", config_disp_bool(cfg->PROXY_PROXY_LINE));
-	fprintf(out, "      --sni-nomatch-abort    Abort handshake when client "
-			"submits an unrecognized SNI server name\n" );
-	fprintf(out, "                             (Default: %s)\n",
+	fprintf(out, "\t--daemon[=on|off]\n");
+	fprintf(out, "\t\tFork into background and become a daemon (Default: %s)\n",
+	    config_disp_bool(cfg->DAEMONIZE));
+	fprintf(out, "\t--write-ip[=on|off]\n");
+	fprintf(out,
+	    "\t\tWrite 1 octet with the IP family followed by the IP\n");
+	fprintf(out,
+	    "\t\taddress in 4 (IPv4) or 16 (IPv6) octets little-endian\n");
+	fprintf(out,
+	    "\t\tto backend before the actual data\n");
+	fprintf(out,
+	    "\t\t(Default: %s)\n", config_disp_bool(cfg->WRITE_IP_OCTET));
+	fprintf(out, "\t--write-proxy-v1[=on|off]\n");
+	fprintf(out,
+	    "\t\tWrite HAProxy's PROXY v1 (IPv4 or IPv6) protocol line\n");
+	fprintf(out, "\t\tbefore actual data\n");
+	fprintf(out, "\t\t(Default: %s)\n",
+	    config_disp_bool(cfg->WRITE_PROXY_LINE_V1));
+	fprintf(out, "\t--write-proxy-v2[=on|off]\n");
+	fprintf(out, "\t\tWrite HAProxy's PROXY v2 binary (IPv4 or IPv6)\n");
+	fprintf(out, "\t\t protocol line before actual data\n");
+	fprintf(out, "\t\t(Default: %s)\n",
+	    config_disp_bool(cfg->WRITE_PROXY_LINE_V2));
+	fprintf(out, "\t--write-proxy[=on|off]\n");
+	fprintf(out, "\t\tEquivalent to --write-proxy-v2. For PROXY \n");
+	fprintf(out, "\t\tversion 1 use --write-proxy-v1 explicitly\n");
+	fprintf(out, "\t--proxy-proxy[=on|off]\n");
+	fprintf(out, "\t\tProxy HAProxy's PROXY (IPv4 or IPv6) protocol\n");
+	fprintf(out, "\t\tbefore actual data (PROXYv1 and PROXYv2)\n");
+	fprintf(out, "\t\t(Default: %s)\n",
+	    config_disp_bool(cfg->PROXY_PROXY_LINE));
+	fprintf(out, "\t--sni-nomatch-abort[=on|off]\n");
+	fprintf(out, "\t\tAbort handshake when client submits an\n");
+	fprintf(out, "\t\tunrecognized SNI server name\n" );
+	fprintf(out, "\t\t(Default: %s)\n",
 			config_disp_bool(cfg->SNI_NOMATCH_ABORT));
-	fprintf(out, "      --ocsp-dir=DIR         Set OCSP staple cache directory\n");
-	fprintf(out, "                             This enables automated retrieval and stapling of OCSP responses\n");
-	fprintf(out, "                             (Default: \"%s\")\n", config_disp_str(cfg->OCSP_DIR));
+	fprintf(out, "\t--alpn-protos=LIST\n");
+	fprintf(out, "\t\tSets the protocols for ALPN/NPN negotiation,\n");
+	fprintf(out, "\t\tprovided as a list of comma-separated tokens\n");
+	fprintf(out, "\t--ocsp-dir=DIR\n");
+	fprintf(out, "\t\tSet OCSP staple cache directory\n");
+	fprintf(out, "\t\tThis enables automated retrieval and stapling\n"
+	    "\t\tof OCSP responses\n");
+	fprintf(out, "\t\t(Default: \"%s\")\n", config_disp_str(cfg->OCSP_DIR));
 	fprintf(out, "\n");
-	fprintf(out, "  -t  --test                 Test configuration and exit\n");
-	fprintf(out, "  -p  --pidfile=FILE         PID file\n");
-	fprintf(out, "  -V  --version              Print program version and exit\n");
-	fprintf(out, "  -h  --help                 This help message\n");
+	fprintf(out, "\t-t  --test\n");
+	fprintf(out, "\t\tTest configuration and exit\n");
+	fprintf(out, "\t-p  --pidfile=FILE\n");
+	fprintf(out, "\t\tPID file\n");
+	fprintf(out, "\t-V  --version\n");
+	fprintf(out, "\t\tPrint program version and exit\n");
+	fprintf(out, "\t-h  --help\n");
+	fprintf(out, "\t\tThis help message\n");
 
 	config_destroy(cfg);
 }
@@ -1401,7 +1495,7 @@ config_parse_cli(int argc, char **argv, hitch_config *cfg)
 		{ "ssl", 0, &ssl, 1},
 		{ "client", 0, &client, 1},
 		{ CFG_CIPHERS, 1, NULL, 'c' },
-		{ CFG_PREFER_SERVER_CIPHERS, 0, NULL, 'O' },
+		{ CFG_PREFER_SERVER_CIPHERS, 2, NULL, 'O' },
 		{ CFG_BACKEND, 1, NULL, 'b' },
 		{ CFG_FRONTEND, 1, NULL, 'f' },
 		{ CFG_WORKERS, 1, NULL, 'n' },
@@ -1418,25 +1512,27 @@ config_parse_cli(int argc, char **argv, hitch_config *cfg)
 		{ CFG_CHROOT, 1, NULL, 'r' },
 		{ CFG_USER, 1, NULL, 'u' },
 		{ CFG_GROUP, 1, NULL, 'g' },
-		{ CFG_QUIET, 0, NULL, 'q' },
+		{ CFG_QUIET, 2, NULL, 'q' },
 		{ CFG_LOG_FILENAME, 1, NULL, 'l' },
 		{ CFG_LOG_LEVEL, 1, NULL, 'L' },
-		{ CFG_SYSLOG, 0, NULL, 's' },
+		{ CFG_SYSLOG, 2, NULL, 's' },
 		{ CFG_SYSLOG_FACILITY, 1, NULL, CFG_PARAM_SYSLOG_FACILITY },
 		{ CFG_SEND_BUFSIZE, 1, NULL, CFG_PARAM_SEND_BUFSIZE },
 		{ CFG_RECV_BUFSIZE, 1, NULL, CFG_PARAM_RECV_BUFSIZE },
 #ifdef TCP_FASTOPEN_WORKS
-		{ CFG_TFO, 0, &cfg->TFO, 1 },
+		{ CFG_TFO, 2, NULL, 1 },
 #endif
-		{ CFG_DAEMON, 0, &cfg->DAEMONIZE, 1 },
-		{ CFG_WRITE_IP, 0, &cfg->WRITE_IP_OCTET, 1 },
-		{ CFG_WRITE_PROXY_V1, 0, &cfg->WRITE_PROXY_LINE_V1, 1 },
-		{ CFG_WRITE_PROXY_V2, 0, &cfg->WRITE_PROXY_LINE_V2, 1 },
-		{ CFG_WRITE_PROXY, 0, &cfg->WRITE_PROXY_LINE_V2, 1 },
-		{ CFG_PROXY_PROXY, 0, &cfg->PROXY_PROXY_LINE, 1 },
+		{ CFG_DAEMON, 2, NULL, 1 },
+		{ CFG_WRITE_IP, 2, NULL, 1 },
+		{ CFG_WRITE_PROXY_V1, 2, NULL, 1 },
+		{ CFG_WRITE_PROXY_V2, 2, NULL, 1 },
+		{ CFG_WRITE_PROXY, 2, NULL, 1 },
+		{ CFG_PROXY_PROXY, 2, NULL, 1 },
 		{ CFG_ALPN_PROTOS, 1, NULL, CFG_PARAM_ALPN_PROTOS },
-		{ CFG_SNI_NOMATCH_ABORT, 0, &cfg->SNI_NOMATCH_ABORT, 1 },
+		{ CFG_SNI_NOMATCH_ABORT, 2, NULL, 1 },
 		{ CFG_OCSP_DIR, 1, NULL, 'o' },
+		{ CFG_TLS_PROTOS, 1, NULL, CFG_PARAM_TLS_PROTOS },
+		{ CFG_DBG_LISTEN, 1, NULL, CFG_PARAM_DBG_LISTEN },
 		{ "test", 0, NULL, 't' },
 		{ "version", 0, NULL, 'V' },
 		{ "help", 0, NULL, 'h' },
@@ -1490,19 +1586,23 @@ config_parse_cli(int argc, char **argv, hitch_config *cfg)
 		case CFG_PARAM_CFGFILE:
 			/* Handled above */
 			break;
-#define CFG_ARG(opt, key)							\
-		case opt:							\
-			ret = config_param_validate(key, optarg, cfg, NULL, 0);	\
+#define CFG_ARG(opt, key)						\
+			case opt:					\
+			ret = config_param_validate(key,		\
+			    optarg, cfg, NULL, 0);			\
 			break;
-#define CFG_ON(opt, key)							\
-		case opt:							\
-			ret = config_param_validate(key, CFG_BOOL_ON, cfg, 	\
-			    NULL, 0);						\
+#define CFG_BOOL(opt, key)						\
+			case opt:					\
+			ret = config_param_validate(key,		\
+			    optarg ? optarg : CFG_BOOL_ON,		\
+			    cfg, NULL, 0);				\
 			break;
 CFG_ARG(CFG_PARAM_SYSLOG_FACILITY, CFG_SYSLOG_FACILITY);
 CFG_ARG(CFG_PARAM_SEND_BUFSIZE, CFG_SEND_BUFSIZE);
 CFG_ARG(CFG_PARAM_RECV_BUFSIZE, CFG_RECV_BUFSIZE);
 CFG_ARG(CFG_PARAM_ALPN_PROTOS, CFG_ALPN_PROTOS);
+CFG_ARG(CFG_PARAM_TLS_PROTOS, CFG_TLS_PROTOS);
+CFG_ARG(CFG_PARAM_DBG_LISTEN, CFG_DBG_LISTEN);
 CFG_ARG('c', CFG_CIPHERS);
 CFG_ARG('e', CFG_SSL_ENGINE);
 CFG_ARG('b', CFG_BACKEND);
@@ -1522,13 +1622,31 @@ CFG_ARG('r', CFG_CHROOT);
 CFG_ARG('u', CFG_USER);
 CFG_ARG('g', CFG_GROUP);
 CFG_ARG('o', CFG_OCSP_DIR);
-CFG_ON('O', CFG_PREFER_SERVER_CIPHERS);
-CFG_ON('q', CFG_QUIET);
+CFG_BOOL('O', CFG_PREFER_SERVER_CIPHERS);
+CFG_BOOL('q', CFG_QUIET);
 CFG_ARG('l', CFG_LOG_FILENAME);
 CFG_ARG('L', CFG_LOG_LEVEL);
-CFG_ON('s', CFG_SYSLOG);
+CFG_BOOL('s', CFG_SYSLOG);
 #undef CFG_ARG
-#undef CFG_ON
+#undef CFG_BOOL
+		case 1:
+			assert (option_index > 0);
+			if (optarg != NULL) {
+				if (strcmp(optarg, "on") &&
+				    strcmp(optarg, "off")) {
+					config_error_set(
+						"Invalid argument '%s' for option '%s': "
+						    "expected one of 'on' or 'off",
+						    optarg,
+						    long_options[option_index].name);
+					return (1);
+				}
+			}
+			ret = config_param_validate(
+				long_options[option_index].name,
+				    optarg ? optarg : CFG_BOOL_ON,
+				    cfg, NULL, 0);
+			break;
 		case 't':
 			cfg->TEST = 1;
 			break;
@@ -1624,18 +1742,6 @@ CFG_ON('s', CFG_SYSLOG);
 				    cfg->ALPN_PROTOS);
 			return (1);
 		}
-		AN(cfg->ALPN_PROTOS_LV);
-		int multi_proto =
-		    cfg->ALPN_PROTOS_LV[0] != cfg->ALPN_PROTOS_LV_LEN - 1;
-		if (multi_proto && !cfg->WRITE_PROXY_LINE_V2) {
-			config_error_set("alpn-protos is specified with"
-			    " more than one protocol while proxy-v2 is "
-			    " not selected. This is a configuration"
-			    " error.");
-			return (1);
-			/* Note that this test was carried out indepenently of
-			   the availability of ALPN / NPN */
-		}
 #if defined(OPENSSL_WITH_NPN) || defined(OPENSSL_WITH_ALPN)
 		/*
 		if (cfg->WRITE_PROXY_LINE_V2)
@@ -1649,6 +1755,9 @@ CFG_ON('s', CFG_SYSLOG);
 		    " support.\n");
 #  endif
 #else
+		AN(cfg->ALPN_PROTOS_LV);
+		int multi_proto =
+		    cfg->ALPN_PROTOS_LV[0] != cfg->ALPN_PROTOS_LV_LEN - 1;
 		/* No support for ALPN / NPN support in OpenSSL */
 		if (multi_proto ||
 		    0 != strncmp((char *)cfg->ALPN_PROTOS_LV, "\x8http/1.1", 9)) {
